import unittest
from catscore.http.web_driver import CatsWebDriver
import argparse
import time
from catscore.lib.functional import calc_time

class TestCatsWebDriver(unittest.TestCase):
    binary_location = "/Applications/Google Chrome Canary.app/Contents/MacOS/Google Chrome Canary"
    executable_path = "/Users/rv/workspace/docker/finance_db/tools/chromedriver"
    proxy = None
    headless = True
    
    def test_simple_get(self):
        """[python -m unittest tests.http.test_web_driver.TestCatsWebDriver.test_simple_get]
        """
        request:CatsRequest = CatsWebDriver(binary_location=self.binary_location, executable_path=self.executable_path, proxy=self.proxy, headless=self.headless)
        html = request.move(url="https://yahoo.co.jp")
        time.sleep(10)
        request.close()
        
    @calc_time()
    def cal_log_time(self, request):
        return request.log


    def test_websocket(self):
        """[python -m unittest tests.http.test_web_driver.TestCatsWebDriver.test_websocket]
        """
        request:CatsRequest = CatsWebDriver(binary_location=self.binary_location, executable_path=self.executable_path, proxy=self.proxy, headless=self.headless)
        html = request.move(url="https://demo.click-sec.com/ixop/order.do")
        time.sleep(2)
        log = self.cal_log_time(request)
        with open("/Users/rv/Desktop/a.text", mode='w') as f:
            for l in log:
                f.write(l)
        request.close()

if __name__ == "__main__":
    unittest.main()


import logging
import requests
from requests import Session
import json
from bs4 import BeautifulSoup
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from abc import ABCMeta, abstractmethod
from catscore.http.request import CatsRequest
from selenium.webdriver.common.desired_capabilities import DesiredCapabilities

class CatsWebDriver:
    def __init__(self, binary_location, executable_path, proxy, headless):
        """[summary]

        Arguments:
            binary_location {[type]} -- [description]
            executable_path {[type]} -- [description]
            proxy {[type]} -- [description]
            headless {[type]} -- [description]
        """
        options = Options()
        options.binary_location = binary_location
        logging.info(f"WebDriverSession.__init__ : {binary_location}, {executable_path}, {proxy}, {headless}")
        if headless:
            options.add_argument('--headless')
            # https://www.ytyng.com/blog/ubuntu-chromedriver/
            options.add_argument("--disable-dev-shm-usage") # overcome limited resource problems
            options.add_argument("start-maximized") # open Browser in maximized mode
            options.add_argument("disable-infobars") # disabling infobars
            options.add_argument("--disable-extensions") # disabling extensions
            options.add_argument("--disable-gpu") # applicable to windows os only
            options.add_argument("--no-sandbox") # Bypass OS security model
        if proxy:
            logging.info("WebDriverSession proxy on")
            options.add_argument(f"proxy-server={proxy}")
            
        caps = DesiredCapabilities.CHROME
        caps['loggingPrefs'] = {'performance': 'INFO'}
        self.driver = webdriver.Chrome(options=options, executable_path=executable_path,desired_capabilities=caps)
        self.driver.implicitly_wait(5)

    def close(self):
        """[Close WebDriverSession, if chromewebdriver dosen't kill, plsease execute "killall chromedriver"]
        
        """
        self.driver.quit()

    def reload(self):
        self.driver.refresh()

    @property
    def cookies(self):
            return self.driver.get_cookies()

    def to_request_session(self) -> CatsRequest: 
        """[summary]
        
        Returns:
            CatsRequest -- [description]
        """
        session = CatsRequest()
        for cookie in self.driver.get_cookies():
            self.driver.cookies.set(cookie["name"], cookie["value"])
        return session

    def wait_rendering_by_id(self, id, timeout=20):
        """[summary]
        
        Arguments:
            id {[type]} -- [description]
        
        Keyword Arguments:
            timeout {int} -- [description] (default: {20})
        """
        WebDriverWait(self.driver, timeout).until(EC.presence_of_element_located((By.ID, id)))

    def wait_rendering_by_class(self, _class, timeout=20):
        """[summary]
        
        Arguments:
            _class {[type]} -- [description]
        
        Keyword Arguments:
            timeout {int} -- [description] (default: {20})
        """
        WebDriverWait(self.driver, 20).until(EC.presence_of_element_located((By.CLASS_NAME, _class)))

    def move(self, url):
        self.driver.get(url)

    @property
    def html(self):
        html = self.driver.page_source.encode('utf-8')
        return BeautifulSoup(html, "lxml")

    @property
    def log(self):
        result = []
        for entry in self.driver.get_log('performance'):
            result.append(entry['message'])
        return result



import logging
import logging.handlers
from datetime import datetime
import os
import errno
import logging
import smtplib
from email.mime.text import MIMEText
from email.utils import formatdate
import socket

class Gmail:
    smtp_url = 'smtp.gmail.com'
    smtp_port = 587

    def __init__(self, username, password, to_address):
        """[summary]

        Arguments:
            username {[type]} -- [description]
            password {[type]} -- [description]
            to_address {[type]} -- [description]
        """
        self.username = username
        self.password = password
        self.to_address = to_address

    def login(self):
        """[summary]
        """
        self.smtp = smtplib.SMTP(self.smtp_url, self.smtp_port)
        self.smtp.ehlo()
        self.smtp.starttls()
        self.smtp.ehlo()
        self.smtp.login(self.username, self.password)

    def send(self, body, subject):
        msg = MIMEText(body)
        msg['Subject'] = subject
        msg['From'] = self.username
        msg['To'] = self.conf.to_address
        msg['Date'] = formatdate()
        self.smtp.sendmail(self.username, self.to_address, msg.as_string())

    def close(self):
        self.smtp.close()

class CatsLogging:
    """[summary]
    """

    datefmt = '%d-%m-%Y:%H:%M:%S'
    format = '%(asctime)s,%(msecs)d %(levelname)-8s [%(filename)s:%(lineno)d] %(message)s'

    @classmethod
    def init(cls, app_name, ouput_dir, log_level, gmail: Gmail=None):
        """[Init logger]
        
        Arguments:
            app_name {[type]} -- [description]
            ouput_dir {[type]} -- [description]
            log_level {[type]} -- [debug/info/error/critical]
        
        Keyword Arguments:
            gmail {Gmail} -- [Send logmessage over error level] (default: {None})
        """
        if log_level == "debug":
            level = logging.DEBUG
        elif log_level == "info":
            level = logging.INFO
        elif log_level== "error":
            level = logging.ERROR
        elif log_level == "critical":
            level = logging.CRITICAL
        else:
            level = logging.INFO

        logging.basicConfig(level=level,
                            filename=f"{ouput_dir}/{app_name}.log",
                            format=cls.format,
                            datefmt=cls.datefmt)
        logging.getLogger('sqlalchemy.engine').setLevel(level)
        if gmail:
                try:
                    gmail.login()
                    handler = logging.handlers.SMTPHandler(mailhost=(gmail.smtp_url, gmail.smtp_port),
                                                        fromaddr=gmail.username,
                                                        toaddrs=gmail.to_address,
                                                        subject=app_name,
                                                        credentials=(gmail.username, gmail.password),
                                                        secure=())
                    handler.setLevel(logging.ERROR)
                    logger = logging.getLogger('root')
                    logger.addHandler(handler)
                except smtplib.SMTPAuthenticationError:
                    cls.error("Cannot login to google")

    @classmethod
    def __add_message_option(cls, mesasge):
        host_name = socket.gethostname()
        ip_addr  =socket.gethostbyname(socket.gethostname())
        return f"{host_name},{ip_addr},{mesasge}"

    @classmethod
    def debug(cls, message):
        #logger = logging.getLogger('root')
        #print(message)
        logging.debug(cls.__add_message_option(message))

    @classmethod
    def info(cls, message):
        #logger = logging.getLogger('root')
        logging.info(cls.__add_message_option(message))

    @classmethod
    def error(cls, message):
        logger = logging.getLogger('root')
        logger.error(cls.__add_message_option(message))

    @classmethod
    def critical(cls, message):
        logger = logging.getLogger('root')
        logger.critical(cls.__add_message_option(message))


from datetime import datetime, timezone
import email.utils as eut

def get_current_time():
    now = datetime.now().strftime("%Y%m%d%H%M%S")
    return now

def get_today_date():
    now = datetime.now().strftime("%Y%m%d")
    return now

def parse_http_date(s):
    """[parse http data format "Wed, 06 Nov 2019 06:53:33 GMT" to datetime]

    Arguments:
        s {[type]} -- [description]
    
    Returns:
        [type] -- [description]
    """
    return datetime(*eut.parsedate(s)[:6], tzinfo=timezone.utc)


from dataclasses import dataclass
from typing import List, Mapping
from bs4 import BeautifulSoup
import json


@dataclass
class Response:
    """[summary]
    """
    headers: Mapping[str, str]
    content: str

@dataclass
class ResponseHtml:
    """[summary]
    """
    headers: Mapping[str, str]
    content: BeautifulSoup

@dataclass
class ResponseJson:
    """[summary]
    """
    headers: Mapping[str, str]
    content:  json


import logging
import requests
from requests import Session
import json
from bs4 import BeautifulSoup
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.common.by import By
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from abc import ABCMeta, abstractmethod
from catscore.http.error import CatsRequestSessionError
from catscore.http.response import Response, ResponseHtml, ResponseJson

class CatsRequest:
    def __init__(self):
        """[summary]
        """
        self.session = requests.Session()

    def close(self):
        """[summary]
        """
        self.session.close()

    def get_cookie(self, key):
        return self.session.cookies.get(key)

    def get_cookies(self):
        return self.session.cookies.get_dict()

    def _mk_result(self, ret, response_content_type):
        if response_content_type == "html":
            soup = BeautifulSoup(ret.content, features="html.parser")
            return ResponseHtml(ret.headers, soup)
        elif response_content_type == "json":
            soup = BeautifulSoup(ret.content, features="html.parser")
            return ResponseJson(ret.headers, json.loads(str(soup)))
        else:
            return Response(ret.headers, ret.content)

    def _check_status_code(self, url, status_code):
        if status_code != 200:
            raise CatsRequestSessionError(f"{url} response code is {status_code}")
        return True

    def get(self, url, response_content_type=None, proxy=None):
        """[summary]
        
        Arguments:
            url {[type]} -- [description]
        
        Keyword Arguments:
            response_content_type {[type]} -- [html or json] (default: {None})
            proxy {[type]} -- [description] (default: {None})
        
        Raises:
            RuntimeError: [description]
        
        Returns:
            [type] -- [description]
        """
        if proxy:
            ret = self.session.get(url, proxies= self.burpProxies, verify=False)
        else:
            ret = self.session.get(url)
        self._check_status_code(url, ret.status_code)
        return self._mk_result(ret, response_content_type)

    def post(self, url, post_data, response_content_type, proxy=None):
        if proxy:
            ret = self.session.post(url, post_data, proxies=self.burpProxies, verify=False)
        else:
            ret = self.session.post(url, post_data)
        self._check_status_code(url, ret.status_code)
        return self._mk_result(ret, response_content_type)



import unittest
from catscore.http.request import CatsRequest


class TestCatsRequest(unittest.TestCase):

    def test_get(self):
        request:CatsRequest = CatsRequest()
        responseHtml = request.get(url="https://yahoo.co.jp", response_content_type="html")
        responseJson = request.get(url="https://api.rescala.jp/current_stock_info", response_content_type="json")
        print(responseJson)
        request.close()

if __name__ == "__main__":
    unittest.main()